#ifndef estados_h
#define estados_h

#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <vector>
#include <string>
#include <iostream>

using namespace std;

class Driver;

class Estado
{
protected:
	string nombre_estado;
public:
	Estado();
	// Posibles eventos
	virtual void iniciar(Driver* driver);
	virtual void pasan_dos_segundos(Driver* driver);
	virtual void dato_recibido(Driver* driver);
	virtual void error_lectura(Driver* driver);
	virtual void lectura_OK(Driver* driver);
	virtual void buffer_lleno(Driver* driver);
	virtual void queda_espacio_buffer(Driver* driver);
	virtual void pasan_diez_segundos(Driver* driver);
	virtual void imprime_estado(Driver* driver);
	// Comprueba si en estado correcto
	virtual void check_error(const string& supuesto);
protected:	
	void cambia_estado(Driver* driver, Estado* estado);
};

// Eventos concretos
class Est_inicio : public Estado
{
private:
	static Estado* instancia;
public:
	Est_inicio();
	static Estado* get_instancia();
	virtual void iniciar(Driver* driver);
};

class Est_comprobando : public Estado
{
private:
	static Estado* instancia;
public:
	Est_comprobando();
	static Estado* get_instancia();
	virtual void pasan_dos_segundos(Driver* driver);
	virtual void dato_recibido(Driver* driver);
};

class Est_leyendo : public Estado
{
private:
	static Estado* instancia;
public:
	Est_leyendo();
	static Estado* get_instancia();
	virtual void error_lectura(Driver* driver);
	virtual void lectura_OK(Driver* driver);
};

class Est_almacenando : public Estado
{
private:
	static Estado* instancia;
public:
	Est_almacenando();
	static Estado* get_instancia();
	virtual void buffer_lleno(Driver* driver);
	virtual void queda_espacio_buffer(Driver* driver);
};

class Est_detenido : public Estado
{
private:
	static Estado* instancia;
public:
	Est_detenido();
	static Estado* get_instancia();
	virtual void pasan_diez_segundos(Driver* driver);
};

class Est_final : public Estado
{
private:
	static Estado* instancia;
public:
	Est_final();
	static Estado* get_instancia();
};

// Clase de gestin del driver
class Driver
{
// Declaracin de estados
private:
	Estado*	estado_actual;
public:
	Driver();
	// Eventos que ocurren en el driver
	virtual void iniciar();
	virtual void pasan_dos_segundos();
	virtual void dato_recibido();
	virtual void error_lectura();
	virtual void lectura_OK();
	virtual void buffer_lleno();
	virtual void queda_espacio_buffer();
	virtual void pasan_diez_segundos();
	virtual void imprime_estado();
	void cambia_estado(Estado* estado);
};

#endif