import abc

# Definición de la clase Pieza
class Pieza(metaclass = abc.ABCMeta):
    # Constructor
    def __init__(self, color, altura, anchura):
        self.__color = color
        self.__altura = altura
        self.__anchura = anchura
           
    @abc.abstractmethod
    def mueve_escaque(self):
        pass
    
    def imprime_datos(self):
        print(f"Color: {self.__color}")
        print(f"Altura: {self.__altura}")
        print(f"Anchura: {self.__anchura}")
        
    #Métodos selectores y mutadores    
    @property
    def nombre(self):
        return self._nombre

    @nombre.setter
    def nombre(self, nombre):
        self._nombre = nombre
        
# Clase Peon
class Peon(Pieza):
    def __init__(self, color, altura, anchura):
        super().__init__(color, altura, anchura)
        self._nombre = "Peon7"
        self.x = 7
        self.y = 2
        self.__amenaza_peon = False
            
    # Método sobrecargado
    def mueve_escaque(self):
        self.y += 1
        self.__amenaza_peon = False
    
    # Método sobrecargado
    def imprime_datos(self):
        print(f"¿Se encuentra amenazando el peón?  {'SI' if self.__amenaza_peon else 'NO'}")
        super().imprime_datos()
        
# Clase Torre
class Torre(Pieza):
    def __init__(self, color, altura, anchura):
        super().__init__(color, altura, anchura)
        self._nombre = "Torre1"
        self.x = 1
        self.y = 1
        self.__amenaza_torre = False
            
    # Método sobrecargado
    def mueve_escaque(self):
        self.y += 7
        self.__amenaza_torre = True
    
    # Método sobrecargado
    def imprime_datos(self):
        print(f"¿Se encuentra amenazando la torre?  {'SI' if self.__amenaza_torre else 'NO'}")
        super().imprime_datos()
 
        
# Clase Caballo
class Caballo(Pieza):
    def __init__(self, color, altura, anchura):
        super().__init__(color, altura, anchura)
        self._nombre = "Caballo1"
        self.x = 2
        self.y = 1
        self.__amenaza_caballo = False
            
    # Método sobrecargado
    def mueve_escaque(self):
        self.x += 1
        self.y += 2
        self.__amenaza_caballo = True
    
    # Método sobrecargado
    def imprime_datos(self):
        print(f"¿Se encuentra amenazando el caballo?  {'SI' if self.__amenaza_caballo else 'NO'}")
        super().imprime_datos()
                 

peon = Peon(1,10,5)
torre = Torre(2, 20, 5)
caballo = Caballo(3, 10, 10)

lista_piezas = [peon, torre, caballo]

# Realiza polimorfismo
for pieza in lista_piezas:
    pieza.mueve_escaque()
    pieza.imprime_datos()
        